function [qd,qdnum,qdden] = qd_mat(T,mq,e,inf,y)
%[qd,qdnum,qdden] = qd_mat(T,m,q,e,inf,y)
%By default, this function compute qd which cancel unstable pole
%of transfer function (1-mqqd)pd. (See Ref.)
% Where:
%	T 	= 	matrix representing disturbance lag
%			T= [  0                  0 + 2.0000i   1.0000     <== process disturbance lag of x(2)s+1      
%			      0             4.0000             1.0000];   <== model disturbance lag of 4s+1
%
%	m	=	the string representing model
%			m = '1/((4*s+1))*exp(-1*s)';
%			
%	q	=	the string representing controller
%			q = '(4*s+1)/((e*s+1))';
%
%	e	=	filter time constant
%
%	y	=	spared variable (in case of uncertain model) 
%			used only for searching the best model (y always =[])
%	Inf = big number(not use any more). Now, I use eps instead.
%        where eps is a small floating point number defined by Matlab.
%
% Note : This function must be used together with the mq_gen function.

qd='1';
qdnum=1;
qdden=1;
poles=roots(T);
poles=sort(poles); % Sort the poles.
n=length(poles);
if isempty(mq{1}) | length(poles) == 0
 return
end
A=zeros(n,n);
b=zeros(n,1);
for i=1:n
   s=poles(i);
   [row,col]=size(mq);
   if col == 1
      temp=eval(mq{1});
      cs=temp/(e*s+1)^n;
      if poles(i)==0 | (i>1 & poles(i)==poles(i-1))
         cs_prime=eval(mq{2})*temp/(e*s+1)^n-e*n*cs/(e*s+1);
         b(i)=-cs_prime;
         for j=1:n
            A(i,j)=cs*(n+1-j)*s^(n-j)+cs_prime*s^(n+1-j);
         end
      else
         b(i)=1-cs;
         for j=1:n
            A(i,j)=cs*s^(n+1-j);
         end
      end
   else
      temp=eval(mq{1,1})*eval(mq{1,2}); % pq
      cs=temp/(e*s+1)^n;
      if i>1 & poles(i)==poles(i-1)
         mq_prime=eval(mq{2,1})*eval(mq{1,2})+eval(mq{1,1})*eval(mq{2,2});
         cs_prime=mq_prime/(e*s+1)^n-e*n*cs/(e*s+1);
         b(i)=-cs_prime;
         for j=1:n
            A(i,j)=cs*(n+1-j)*s^(n-j)+cs_prime*s^(n+1-j);
         end
      else
         b(i)=1-cs;
         for j=1:n
            A(i,j)=cs*s^(n+1-j);
         end
      end
   end
end
Alpha=real(inv(A)*b);
%Alpha=inv(A)*b  % This line is for testing to see if imaginary parts are really small.
qd='(';
qdden=[1];
for k=1:n
   if k==n
      qd=[qd num2str(Alpha(k)) '*s+'];
   else
      qd=[qd num2str(Alpha(k)) '*s^' num2str(n+1-k) '+'];
   end
   qdden=conv(qdden,[e 1]);
end
if n==1
   qd=[qd '1)/(e*s+1)'];
else
   qd=[qd '1)/(e*s+1)^' num2str(n)];
end
qdnum=[Alpha' 1];



