function [n_circle,data]=detect_RHPZ(m,T)
% function [n_circle,data]=detect_RHPZ(m)
% detect right half plane of the MIMO system, m. The Nyquist plot has been calculated
% along the D-contour ignoring the poles on the imaginary axis by moving the contour
% little bit to the positive real direction, with a distance = eps which is the smallest
% accuracy represented by Matlab.
% The function plot Nyquist plot and return the number of encirclement around the origin.
f=crfig(50,50,520,400,['Nyquist plot of the determinant of the MIMO model'],...
   'w','none','off');
xlabel('Real');
ylabel('Imaginary');
hold on
grid on
model=cell2str(m);
n_circle=0;
w_step=1.1; % frequency magnification
w=0.001; % starting frequency (small radius of D contour)
w_inf=1000; % stopping frequency (big raduis of D contour)
n_point=1;
% moving along the positive imaginary axis
s=eps+w*sqrt(-1);
value=eval(model);
data(n_point)=det(value)/prod(diag(value*T)); % Evaluate the determinant, and then factor 
n_point=n_point+1;									% out the product of the diagonal element.
while w < w_inf & abs(data(n_point-1))>1e-4
   temp=w;
   w=w*w_step;
   s=eps+w*sqrt(-1);
   value=eval(model);
   data(n_point)=det(value)/prod(diag(value*T));
   if n_point >=3
   temp1 = abs(angle(data(n_point)-data(n_point-1))-angle(data(n_point-1)-data(n_point-2)));
%   adjudge the frequency increment to smoot the curve
   	if temp1 > 0.5 & w_step > 1.001
      	w=temp;
      	w_step=1+(w_step-1)/2;
   	elseif temp1 < 0.1
      	w=temp;
      	w_step=1+(w_step-1)*1.5;
   	else
      	n_point=n_point+1;
   	end
   else
      n_point=n_point+1;
   end
end
% turning clockwise around the big radius
for ceta=pi/2-pi/20:-pi/20:0
   s=w*(cos(ceta)+sin(ceta)*sqrt(-1));
	value=eval(model);
   data(n_point)=det(value)/prod(diag(value*T));
   n_point=n_point+1;
end
data=[conj(fliplr(data)) data];
% ************************************
% Counting the number of encirclement
initial_angle = rem(angle(data(1)) + 2*pi, 2*pi);
angle_data = rem(angle(data) - initial_angle + 4*pi,2*pi);
diff_angle=diff(angle_data);
Ipd = find(diff_angle < -pi);
Ind = find(diff_angle > pi);
n_circle= (length(Ind))-(length(Ipd));
% **************************************
uicontrol(f,...
'Style','text',...
'Position',[150 355 150 23],...
'Backgroundcolor','w',...
'Foregroundcolor','k',...
'HorizontalAlignment','center',...
'String','Number of RHP zeros = ');
uicontrol(f,...
 'Style','frame',...
 'Position',[290 357 100 24],...
 'Foregroundcolor','k');
uicontrol(f,...
 'Style','edit',...
 'Position',[295 360 90 18],...
 'Backgroundcolor','w',...
 'Foregroundcolor','k',...
 'String',num2str(n_circle));
uicontrol(f,...
  'Style','push',...
  'Position',[380 15 80 25],...
  'String','Close',...
 'Callback','close'); 

plot(real(data),imag(data),'b');
temp1=min(real(data));
temp2=max(real(data));
%temp3=max(-min(imag(data)),max(imag(data)));
plot(0,0,'or');
temp3=(temp2-temp1)/20;
[temp,temp1]=min(imag(data));
temp2=data(temp1-1)-data(temp1);
ceta=angle(temp2)+0.3;
temp=data(temp1)+(temp3*cos(ceta)+temp3*sin(ceta)*sqrt(-1));
plot(real([data(temp1) temp]),imag([data(temp1) temp]),'r');
ceta=angle(temp2)-0.3;
temp=data(temp1)+(temp3*cos(ceta)+temp3*sin(ceta)*sqrt(-1));
plot(real([data(temp1) temp]),imag([data(temp1) temp]),'r');
      
      