function [p]=tfn(k,n,d,td)
% tfn takes a scalar gain, k, matricies n and d, and dead time, td
% to form a single input, single output transfer function, using the rows
% of n and d to form the numerator and denominator polynomials, respectively. 
%If the dead time is omitted, it is taken as zero.
% Example: num=[1 3] 
%          den=[1 2 5;0 1 2]
%          td=6
% g= tfn(5,num,den,td)
%  
% forms:
% 
% g=5(s + 3)/((s^2 + 2s + 5)(s + 2)); with a delay of 6
%
% which is actually written out as: 
%
% 1.5*(0.3333 s + 1)
% ------------------------------
% (.2 s^2 + 0.4 s + 1)*( 0.5 s + 1)
%
% input delay: 6
%
% and as:
%
% Transfer function:
%        5 s + 15
%  ----------------------
%  s^3 + 4 s^2 + 9 s + 10
%
% input delay: 6
%
% Notice that each row of num or den must have the same number of elements.
% Therefore,lower order polynomials must have explicit zero coefficients for
% the higher order terms.
 

nin=nargin;
if nin==3 
   [p]=tfn1(k,n,d,0);
elseif nin==4 
   [p]=tfn2(k,n,d,td);
end

